#include <libwccl/ops/match/conditions/optionalmatch.h>
#include <libwccl/values/matchvector.h>
#include <sstream>

namespace Wccl {

OptionalMatch::OptionalMatch(const boost::shared_ptr<ConjConditions>& conditions)
	: _conditions(conditions)
{
	BOOST_ASSERT(_conditions);
}

MatchResult OptionalMatch::apply(boost::shared_ptr<Position> iter_pos, const FunExecContext& context) const
{
	int orig_pos = iter_pos->get_value();
	MatchResult res = _conditions->apply(iter_pos, context);
	if (res.matched()) {
		return res;
	}
	iter_pos->set_value(orig_pos);
	return MatchResult(boost::make_shared<Match>());
}

std::string OptionalMatch::to_string(const Corpus2::Tagset& tagset) const
{
	std::ostringstream ostream;
	ostream << name() << _conditions->to_string(tagset);
	return ostream.str();
}

std::ostream& OptionalMatch::write_to(std::ostream& ostream) const
{
	return ostream << name() << *_conditions;
}

} /* end ns Wccl */
